# Test configuration parsing
# Copyright (C) 2020  Nguyễn Gia Phong
#
# This file is part of Acanban.
#
# Acanban is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Acanban is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Acanban.  If not, see <https://www.gnu.org/licenses/>.

from os.path import abspath, dirname, join
from typing import Any, Mapping, Sequence

import toml
from pytest import mark, param

from acanban.config import hypercorn_config

CONFIG_DIR = abspath(join(dirname(__file__), 'assets'))
EMPTY_DIR = 'this directory does not exist'

HYPERCORN_MOCK = toml.load(join(CONFIG_DIR, 'hypercorn.toml'))
HYPERCORN_DEFAULT = {'bind': ['127.0.0.1:8000']}


@mark.parametrize(('dirs', 'mapping'), (
    param([CONFIG_DIR], HYPERCORN_MOCK, id='load'),
    param([EMPTY_DIR, CONFIG_DIR], HYPERCORN_MOCK, id='load'),
    param([EMPTY_DIR], HYPERCORN_DEFAULT, id='fallback')))
def test_hypercorn_config(dirs: Sequence[str],
                          mapping: Mapping[str, Any]) -> None:
    """Test loading Hypercorn configuration."""
    config = hypercorn_config(*dirs)
    for key, value in mapping.items(): assert getattr(config, key) == value



# Initialization code
# Copyright (C) 2020  Nguyễn Gia Phong
#
# This file is part of Acanban.
#
# Acanban is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Acanban is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Acanban.  If not, see <https://www.gnu.org/licenses/>.

from quart_trio import QuartTrio

from .static import index_html

__all__ = ['app']
__doc__ = 'Academic Kanban'
__version__ = '0.0.1'

app = QuartTrio(__name__)


@app.route('/')
async def index() -> str:
    """Return the index page."""
    return index_html



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options.
# For a full list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

from acanban import __version__

# Project information
project = 'Acanban'
copyright = '2020  Ngô Ngọc Đức Huy et al.'  # noqa
author = 'Ngô Ngọc Đức Huy et al.'
release = __version__

# Add any Sphinx extension module names here, as strings.
# They can be extensions coming with Sphinx (named 'sphinx.ext.*')
# or your custom ones.
extensions = ['sphinx.ext.githubpages', 'sphinxcontrib.plantuml']
plantuml_output_format = 'svg_img'
plantuml_latex_output_format = 'pdf'

# Add any paths that contain templates here, relative to this directory.
templates_path = []

# List of patterns, relative to source directory, that match
# files and directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# Options for HTML output
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets)
# here, relative to this directory.  They are copied after the builtin
# static files, so a file named "default.css" will overwrite the builtin
# "default.css".
html_static_path = []

# Options for LaTeX
latex_elements = {'papersize': 'a4paper', 'pointsize': '12pt',
                  'fontpkg': r'\usepackage{lmodern}'}



# Configuration parser
# Copyright (C) 2020  Nguyễn Gia Phong
#
# This file is part of Acanban.
#
# Acanban is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Acanban is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Acanban.  If not, see <https://www.gnu.org/licenses/>.

from os.path import isfile, join

from hypercorn.config import Config


def hypercorn_config(*directories: str) -> Config:
    """Return Hypercorn configuration first found in given directories."""
    for directory in directories:
        file = join(directory, 'hypercorn.toml')
        if isfile(file): return Config.from_toml(file)
    return Config()



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['acanban', 'acanban.static']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['appdirs', 'hypercorn', 'quart-trio']

setup(name='acanban',
      version='0.0.1',
      description='Academic Kanban',
      author='Ngô Ngọc Đức Huy',
      author_email='duchuy29092000@gmail.com',
      url='https://Huy-Ngo.github.io/acanban',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      python_requires='>=3.7',
     )



# Server console script
# Copyright (C) 2020  Nguyễn Gia Phong
# Copyright (C) 2020  Ngô Ngọc Đức Huy
#
# This file is part of Acanban.
#
# Acanban is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Acanban is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Acanban.  If not, see <https://www.gnu.org/licenses/>.

from appdirs import site_config_dir, user_config_dir
from hypercorn.trio import serve
from trio import run

from . import app
from .config import hypercorn_config

if __name__ == '__main__':
    config = hypercorn_config(user_config_dir(), site_config_dir())
    run(serve, app, config)
